/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "kernel.h"
#include "Parser.h"
#include <cassert>
#include <memory>
#include "stlsoft.h"
#include "stlcatalogs.h"
#include "select_variants.h"
#include "registry_remover.h"
#include "strings_utils.h"

using namespace kr;

TCHAR const * SLASH_CATS = _T("/");		// 
TCHAR const * SLASH_DIRS = _T("\\");	// 
TCHAR const SLASH_CATS_CHAR = _T('/');	// 
TCHAR const SLASH_DIRS_CHAR = _T('\\');	// 

TCHAR const* HISTORY_APPLY_COMMANDS = _T("nf_history_apply_commands");
namespace 
{

	bool expand_catalog_path(tstring const &SrcCatalog
		, tstring TargetCatalog
		, tstring& ResultCatalog)
	{	
		// ".."  TargetCatalog
		//"1/2/3/" ".." -> "1/2/../3" = "1/3"
		//"1/2/3/4/" "../../7" -> "1/2/3/../../7"="1/7"
		//"1/2/3/4/" "../../7" -> "1/2/3/../../7/4"="1/7/4"
		//"1/2/3/" "/5/6" -> "/5/6"
		//"1/2/3/" "/5/6/" -> /5/6/3"
		//"1/2/3/" "5/6" -> 1/2/5/6
		//"1/2/3/" "5/6/" -> 1/2/5/6/3
		const tstring ROOT(_T("../"));
		const tstring ROOT_SHORT(_T("/.."));
		if (! TargetCatalog.size()) return false;

		bool bRelatedRoot = (*(TargetCatalog.begin()) == SLASH_CATS_CHAR);
		bool bRelatedCurrentCatalog = 
			(*(TargetCatalog.end()-1) == SLASH_CATS_CHAR);
		add_trailing_char_if_not_exists(TargetCatalog, SLASH_CATS_CHAR);
		if (TargetCatalog.size() >= ROOT.size())
		{
			tstring::const_reverse_iterator p1 = TargetCatalog.rend();
			tstring::const_reverse_iterator p2 = ROOT.rend();
			while (p2 != ROOT.rbegin())
			{
				if (*p1 == *p2)
				{
					--p1;
					--p2;
				} else break;
			}
			if (p2 == ROOT.rbegin()) 
				bRelatedCurrentCatalog = true;
		}
		
		tstring src_path;
		tstring src_catalog_name;
		divide_path_filename(SrcCatalog, src_path, src_catalog_name, SLASH_CATS_CHAR, true);
		add_trailing_char_if_not_exists(src_path, SLASH_CATS_CHAR);

		if (bRelatedRoot)
		{
			ResultCatalog.swap(TargetCatalog);
		} else 
		{
			ResultCatalog.swap(src_path);
			ResultCatalog.append(TargetCatalog);
		} 
		if (bRelatedCurrentCatalog) 
		{
			remove_trailing_chars(ResultCatalog, SLASH_CATS_CHAR);
			ResultCatalog.append(src_catalog_name);
		}

		//  ..
		add_leading_char_if_not_exists(ResultCatalog, SLASH_CATS_CHAR);
		tstring::size_type npos;
		while ((npos = ResultCatalog.find(ROOT)) != tstring::npos)
		{
			tstring::size_type npos2 = 
				ResultCatalog.find_last_of(SLASH_CATS_CHAR, npos-ROOT.size());
			if ((npos2 == tstring::npos)  || npos2 > npos)
			{
				ResultCatalog = _T("");
				return true;	//  
			}
			ResultCatalog.erase(npos2, npos-npos2+ROOT.size()-1);
		}

		remove_trailing_chars(ResultCatalog, SLASH_CATS_CHAR);
		return true;
	}
} //namespace


CKernel::CKernel()
{
}

CKernel::~CKernel(void)
{
}


bool CKernel::parse_command(tstring const &source, kr::tparsed_command &cmd)
{	//      
	parser::CParser pr;
	return pr.ParseString(source, cmd);
}


// ---  

size_t CKernel::select_variants_catalogs(TCHAR const *cat_pattern, tcatalogs_list& list)
{	//     
	return ::select_variants_catalogs(cat_pattern, list);
}

size_t CKernel::select_variants_shortcuts(tstring shortcut_pattern
										  , tstring catalog
										  , tshortcuts_list& list)
{	//     
	if (!shortcut_pattern.size()) 
		shortcut_pattern = _T("*");
	remove_trailing_chars(catalog, SLASH_CATS_CHAR);
	::select_variants_shortcuts(catalog, shortcut_pattern, list);
	if (list.size())
		return ::sort_by_relevance(list, tstring(catalog), tstring(shortcut_pattern));
	return 0;
}

size_t CKernel::select_variants_catalogs_and_shortcuts(TCHAR const *shortcut_pattern, 
													   TCHAR const* catalog_pattern, 
													   tshortcuts_list& list)
{	//         
	tcatalogs_list clist;
	//  ,  
	::select_variants_catalogs(catalog_pattern, clist);
	//       
	tcatalogs_list::const_iterator p = clist.begin();
	while (p != clist.end())
	{
		::select_variants_shortcuts(shortcut_pattern, p->c_str(), list);
		++p;
	}
	return list.size();
}

size_t CKernel::select_variants_dirs(tstring const &root_dir,  tstring const&dir_pattern, tdirs_list& list)
{	//     
//  \a*\?\c
//  dir      \a*
//      ?      c -   
//    list
	::select_variants_dirs(root_dir.c_str(), dir_pattern.c_str(), list);
	return list.size();
}


// ---  
bool CKernel::insert_shortcut(kr::tshortcut_info const&sh, 
							  tstring value,
							  bool bOverride)
{	//    
	tstring catalog_name = sh.catalog;
	remove_leading_chars(catalog_name, SLASH_CATS_CHAR);
	remove_trailing_chars(catalog_name, SLASH_CATS_CHAR);

	sc::CCatalog c(catalog_name);
	if (! bOverride)
	{
		tstring buf;
		if (c.get_shortcut_info(sh.shortcut, sh.bIsTemporary, buf)) 
			return false; //   ...
	}
	c.set_shortcut(sh.shortcut, value, sh.bIsTemporary);
	return true;
}

bool CKernel::delete_shortcut(tshortcut_info const&sh)
{	// 
	sc::CCatalog c(sh.catalog);
	c.delete_shortcut(sh.shortcut, sh.bIsTemporary);

	return true;
}

bool CKernel::modify_shortcut(tshortcut_info const&from
							 , tshortcut_info const&to
							 , tstring * pnew_value)
{	//  
	tstring value;
	if (! pnew_value)
	{
		get_value_for_shortcut(from, value);
	} else {
		value = *pnew_value;
	}

	g_pK->delete_shortcut(from);
	g_pK->insert_shortcut(to, value, true);

	return true;
}


bool CKernel::insert_catalog(tstring catalog
							 , TCHAR const* Parent)
{
	remove_trailing_chars(catalog, SLASH_CATS_CHAR);

	sc::CCatalog c(Parent);
	return c.insert_subcatalog(catalog);
}

bool CKernel::remove_catalog(tstring const& SourceCatalog
							 , tstring const* pTargetCatalog
							 , bool bDeleteSource)
{
	//         
	//  ;     
	//         
	nf::registry_remover rr;

	tstring src_key = sc::CCatalog(SourceCatalog).catalog_regkey();
	if (! pTargetCatalog) return rr.Erase(src_key);

	tstring target_catalog;
	if (! expand_catalog_path(SourceCatalog, *pTargetCatalog, target_catalog)) return false;

	tstring target_key = sc::CCatalog(target_catalog).catalog_regkey();

	if (bDeleteSource)
	{
		return rr.Move(src_key, target_key);
	} else 
	{
		return rr.Copy(src_key, target_key);
	}
}

bool CKernel::get_value_for_shortcut(tshortcut_info const& sh
									 , tstring& value)
{
	sc::CCatalog c(sh.catalog);
	return c.get_shortcut_info(sh.shortcut, sh.bIsTemporary, value);
}

bool CKernel::move_shortcut(tshortcut_info const& sh
							, tstring const& new_shortcut_path
							, tshortcut_info &sh2
							)
{	//    .
	//    /    -  
	//  -    -   
	//   ..
	
	sh2 = sh;
	if (*new_shortcut_path.rend() != SLASH_CATS_CHAR)
	{	
		tstring name;
		divide_path_filename(new_shortcut_path, sh2.catalog, name
			, SLASH_CATS_CHAR, false);
		remove_leading_chars(name, SLASH_CATS_CHAR);
		if (! name.empty())
			sh2.shortcut = name;
	}

	if (! sh2.catalog.empty())
		if (! expand_catalog_path(sh.catalog, sh2.catalog, sh2.catalog))
			return false;

	return modify_shortcut(sh, sh2, 0);
}

bool CKernel::copy_shortcut(tshortcut_info const& sh
							, tstring const& new_shortcut_path
							, tshortcut_info &sh2)
{	//    .
	//    /    -  
	//  -    -   
	//   ..

	sh2 = sh;
	if (*new_shortcut_path.rend() != SLASH_CATS_CHAR)
	{	
		tstring name;
		divide_path_filename(new_shortcut_path, sh2.catalog, name
			, SLASH_CATS_CHAR, false);
		remove_leading_chars(name, SLASH_CATS_CHAR);
		if (! name.empty())
			sh2.shortcut = name;
	}

	tstring value;
	if (! get_value_for_shortcut(sh, value)) return false;

	if (! sh2.catalog.empty())
		if (! expand_catalog_path(sh.catalog, sh2.catalog, sh2.catalog))
			return false;

	return insert_shortcut(sh2, value, false);
}


tstring CKernel::GetFirstPluginPrefix()
{
	tstring prefix = m_S.GetValue(nf::STS_PREFIXES);
	tstring::size_type npos = prefix.find_first_of(_T(":"));
	if (npos != tstring::npos)
		prefix.erase(npos + 1, prefix.size() - npos + 1);
	return prefix;
}


size_t CKernel::select_variants_shortcuts_by_path(TCHAR const *catalog_pattern
												  , TCHAR const *value_pattern
												  , bool bExactCoincidence
												  , tshortcuts_list& list)
{
	return ::select_variants_shortcuts_by_path(catalog_pattern
		, value_pattern
		, list
		, bExactCoincidence);	
}
