/*
* Far Named Folders 2.x
* Copyright (c) 2002-2004 by Victor Derevyanko, RAMMuS Group
* www: http://www.rammus.ru
* e-mail: wingspan@yandex.ru
*/
#include "StdAfx.h"
#include "executor_menus.h"
#include <vector>
#include <cassert>
#include "stlcatalogs.h"
#include "shlwapi.h"

using namespace nf;

#pragma comment(lib, "shlwapi.lib")

namespace nf
{
	enum menu_modes_t//      
	{
		MM_S		=	1		// 
		, MM_V		=	2		// 
		, MM_SV		=	3		//  
		, MM_CS		=	4		//  
		, MM_CSV	=	5		//,   
	};
}

inline void safe_copy(char t[128], tstring const &s1)
{
	if (s1.size() > 127)
	{
		PathCompactPathEx(t, s1.c_str(), 127, 0);
	} else 
		lstrcpy(t, s1.c_str());
}

inline void safe_copy(char t[128], tstring const &s1, tstring const &s2, tstring *ps3, TCHAR chdelim1 = _T(' '))
{
	tstring s;
	s.reserve(s1.size() + s2.size() + (ps3 ? ps3->size() : 0) + 2);
	s = s1;
	if (s.size() > 0) s = s + chdelim1;
	s = s + s2;
	if (s.size() > 0) s = s + _T(" ");
	if (ps3) s = s + *ps3;
	PathCompactPathEx(t, s.c_str(), 127, 0);
}

void load_shortcuts_values(kr::tshortcuts_list const &list, kr::tvalues_list &values)
{	//      list
	values.clear();

	kr::tshortcuts_list::const_iterator p1 = list.begin();
	while (p1 != list.end())
	{
		kr::tshortcut_info const &sh = *p1;
		tstring value;
		sc::CCatalog c(sh.catalog);
		if (c.get_shortcut_info(sh.shortcut, sh.bIsTemporary, value))
		{
			values.push_back(value);
		} else {
			assert(false);
			values.push_back(_T(""));
		}
		++p1;
	}
}

//   
//   
int nf::select_required_shortcut_through_menu(kr::tshortcuts_list const &list
											  , kr::tvalues_list &values
											  , kr::tshortcut_info& sh)
{	
	if (! list.size()) 
		return 0;		//  

	if (1 == list.size()) 
	{
		sh = *list.begin();
		return 1;		//  -    ..
	}

	int CurrentMenuMode = g_pK->m_S.GetValue(nf::ST_SELECT_SH_MENU_MODE);	//   
	BOOL bShowKeysInMenu =  g_pK->m_S.GetValue(nf::ST_SHOW_KEYS_IN_MENU);	//    

	while (true)
	{
		const NUM_BREAK_KEYS = 8;
		int BreakKeys[NUM_BREAK_KEYS] =	//     	
			{VK_DELETE, VK_F2, VK_F3, VK_F4, VK_F5, VK_F6, VK_SPACE, 0};	

		switch(CurrentMenuMode) {
		case MM_CSV:
		case MM_V:
		case MM_SV:
			//   
			if (list.size() != values.size()) //    
				load_shortcuts_values(list, values);	//
		};

		std::vector<FarMenuItem> MenuItems(list.size());
		for (unsigned int i = 0; i < list.size(); ++i)
		{
			switch (CurrentMenuMode)
			{
			case MM_V:
				safe_copy(MenuItems[i].Text, values[i]);
				break;
			case MM_SV:
				safe_copy(MenuItems[i].Text, list[i].shortcut, values[i], 0);
				break;
			case MM_CSV:
				safe_copy(MenuItems[i].Text, list[i].catalog, list[i].shortcut, &values[i], _T('/'));
				break;
			case MM_CS:
				safe_copy(MenuItems[i].Text, list[i].catalog, list[i].shortcut, 0, _T('/'));
				break;
			default:
				safe_copy(MenuItems[i].Text, list[i].shortcut);
				break;
			};//
			MenuItems[i].Selected = 0;
			MenuItems[i].Checked = 0;
			MenuItems[i].Separator = 0;
		};

		int BreakCode;

		int nSelectedItem = g_PluginInfo.Menu(g_PluginInfo.ModuleNumber
			, -1
			, -1
			, 0	// -  
			, FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE | FMENU_SHOWAMPERSAND
			, ""
			, (bShowKeysInMenu ? "Del, F2-F6" : "")
			, "MenuSelect"
			, BreakKeys
			, &BreakCode
			, &MenuItems[0]
			, static_cast<int>(MenuItems.size())
		);

		if (-1 == nSelectedItem) return 0;	//user has canceled menu

		if (BreakCode == -1)
		{	//  
			sh = list[nSelectedItem];
			break;
//		} else if (BreakCode == 0)
//		{	//Space -     

		} else if (BreakCode == 0)
		{	//Del -   
			sh = list[nSelectedItem];
			//    
			g_pK->m_S.SetValue(nf::ST_SELECT_SH_MENU_MODE, CurrentMenuMode);	
			return -1;
		} else {//   
			CurrentMenuMode = BreakCode;
			//    
			g_pK->m_S.SetValue(nf::ST_SELECT_SH_MENU_MODE, CurrentMenuMode);	
		}
	}; //while

	return 1;
}


int nf::select_required_catalog_through_menu(kr::tcatalogs_list const &list	//  
											 , kr::tcatalog_info &c 		// 
											 )
{
	if (! list.size()) 
		return 0;		//  

	if (1 == list.size()) 
	{
		c = *list.begin();
		return 1;		//  -    ..
	}

	std::vector<FarMenuItem> MenuItems(list.size());
	for (unsigned int i = 0; i < list.size(); ++i)
	{
		safe_copy(MenuItems[i].Text, list[i]);
		MenuItems[i].Selected = 0;
		MenuItems[i].Checked = 0;
		MenuItems[i].Separator = 0;
	}

	int nSelectedItem = g_PluginInfo.Menu(g_PluginInfo.ModuleNumber
		, -1
		, -1
		, 0	// -  
		, FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE
		, ""
		, ""
		, ""
		, 0
		, 0
		, &MenuItems[0]
		, static_cast<int>(MenuItems.size())
	);

	if (-1 == nSelectedItem) return 0;	//user has canceled menu
	
	c = list[nSelectedItem];

	return 1;
}


bool nf::select_required_path_through_menu (kr::tdirs_list const &list
											, tstring &result_path
											)
{
	if (! list.size()) return 0;		

	if (1 == list.size())  
	{
		result_path = *list.begin();
		return true;
	}

	std::vector<FarMenuItem> MenuItems(list.size());
	for (unsigned int i = 0; i < list.size(); ++i)
	{
		safe_copy(MenuItems[i].Text, list[i]);
		MenuItems[i].Selected = 0;
		MenuItems[i].Checked = 0;
		MenuItems[i].Separator = 0;
	}

	int nSelectedItem = g_PluginInfo.Menu(g_PluginInfo.ModuleNumber
		, -1
		, -1
		, 0	// -  
		, FMENU_AUTOHIGHLIGHT | FMENU_WRAPMODE
		, ""
		, ""
		, ""
		, 0
		, 0
		, &MenuItems[0]
		, static_cast<int>(MenuItems.size())
			);

		if (-1 == nSelectedItem) return 0;	//user has canceled menu

		result_path = list[nSelectedItem];

		return 1;
}
